%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Multi-stage Dantzig selector
% reference: Multi-stage Dantzig Selector, NIPS 2010
% by: Ji Liu
% date: 09/2009
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% input
% F_0:  the initial support set, it could be empty if one has no idea of it
% X:  the design matrix\
% y:  the observation
% lambda: the parameter in the standard Dantzig selector 
% N: the number of iteration
% beta:  the initial signal, it could be empty if one has no idea of it

%% output
% betaList:  return the list of beta in each iteration, the last beta is
% the real output of Multi-stage Dantzig selector
% F0:  the estimated support set whose size is N
% exitMsg:  it returns the exit message in calling the matlab function
% linprog

%% function
function [betaList, F0, exitMsg] = MultiStageDantzigSelector(F0,...
X,...
y,...
lambda,...
N,...
beta)

[n, m] = size(X);
bar_F0 = setdiff([1:m], F0);

if isempty(beta)
    v = [];
else
    v = [abs(beta(bar_F0)); beta];
end

warning('No');
exitMsg = [];
betaList = zeros(m,N+1);
for i = 0:N
    if ~isempty(v)
        [a1, a2] = sort(abs(beta), 'descend');
        F0 = sort(a2(1:1+length(F0)));
        bar_F0 = setdiff([1:m], F0);
        v = [abs(beta(bar_F0)); beta];
    end
    
    f = [ones(length(bar_F0), 1); zeros(m, 1)];
    Aeq = [zeros(length(F0), length(bar_F0)), X(:,F0)'*X];
    beq = [X(:,F0)'*y]; I_m = eye(m);
    A = [-eye(length(bar_F0)), zeros(length(bar_F0), m);
        -eye(length(bar_F0)), I_m(bar_F0, :);
        -eye(length(bar_F0)), -I_m(bar_F0, :);
        zeros(length(bar_F0)), X(:,bar_F0)'*X;
        zeros(length(bar_F0)), -X(:,bar_F0)'*X];
    options = optimset('Largescale', 'off', 'Algorithm', 'active-set', 'MaxIter',100000, 'DiffMinChange',1e-6);
    %options = optimset('MaxIter',500000, 'DiffMinChange',1e-6);
    b =  [zeros(3*length(bar_F0),1); lambda+X(:,bar_F0)'*y; lambda-X(:,bar_F0)'*y];

    [v,fval,exitflag,output] = linprog(f, A, b, Aeq, beq, [], [], v, options);
%     exitMsg = lastwarn;
%     if ~strcmp(exitMsg, 'No')
%         return;
%     end
    beta = v(end-m+1:end);
    betaList(:,i+1) = beta;
end

